new;
library tsm,optmum,spt,pgraph;
TSmset;

cls;

rndseed 123;

a = 0.95; b = 0.10; sigma = 0.2; lambda = 0.05;
theta = 0.5;

bprime = b - lambda*sigma/a;


proc (2) = Theta_Euler(r0,t0,T,N,M,theta);
  local k,tau,u,S,Si,w1,w2,w3,i;

  k = (T - t0)/(N-1);
  tau = seqa(0,k,N);
  u = rndn(N,M)*sqrt(k);
  S = zeros(N,M);

  Si = r0.*ones(1,M);
  S[1,.] = Si;

  w1 = 1 + theta*a*k;
  w2 = 1 - (1-theta)*a*k;
  w3 = a*k*bprime;

  i = 2;
  do until i > N;
    Si = ( w2*Si + w3 + sigma*u[i,.] )/w1;
    S[i,.] = Si;
    i = i + 1;
  endo;

  retp(tau,S);
endp;


proc intTrapeze(S,k);
  local N,I;

  N = rows(S);
  I = k*(sumc(S[2:N-1,.]) + 0.5*(S[1,.]' + S[N,.]') );

  retp(I);
endp;


proc intSimpson(S,k);
  local N,w,I;

  N = rows(S);
  w = 1|reshape(4~2,1,n-2)'|1;
  I = k*(w'S)/3;
  retp(I');
endp;


t0 = 0;
T = 1;
r0 = 0.10;

M = 250; 

N1 = 26;
k1 = (T-t0)/(N1-1);

N2 = 51;
k2 = (T-t0)/(N2-1);

N3 = 101;
k3 = (T-t0)/(N3-1);

N4 = 251;
k4 = (T-t0)/(N4-1);

{tau,S} = Theta_Euler(r0,t0,T,N1,M,theta);
P1a = exp(-intTrapeze(S,k1));
P1b = exp(-intSimpson(S,k1));

{tau,S} = Theta_Euler(r0,t0,T,N2,M,theta);
P2a = exp(-intTrapeze(S,k2));
P2b = exp(-intSimpson(S,k2));

{tau,S} = Theta_Euler(r0,t0,T,N3,M,theta);
P3a = exp(-intTrapeze(S,k3));
P3b = exp(-intSimpson(S,k3));

{tau,S} = Theta_Euler(r0,t0,T,N4,M,theta);
P4a = exp(-intTrapeze(S,k4));
P4b = exp(-intSimpson(S,k4));




{x1a,d1a,F1a,retcode} = Kernel(P1a);
{x1b,d1b,F1b,retcode} = Kernel(P1b);
{x2a,d2a,F2a,retcode} = Kernel(P2a);
{x2b,d2b,F2b,retcode} = Kernel(P2b);
{x3a,d3a,F3a,retcode} = Kernel(P3a);
{x3b,d3b,F3b,retcode} = Kernel(P3b);
{x4a,d4a,F4a,retcode} = Kernel(P4a);
{x4b,d4b,F4b,retcode} = Kernel(P4b);

{CouponZero,TauxZero} = Vasicek(r0,a,b,sigma,lambda,T-t0);

graphset;
  begwind;
  window2(2,2,0,0.75);

  setwind(1);

    _pdate = ""; _ptitlht = 1.5; _pnum = 0; _paxes = 0;
    fonts("simplex simgrma complex");
    title("Comparaison des algorithmes numeriques d'integration"\
          "\Let influence du pas de discretisation");
    draw;

  graphset;
    _pdate = ""; _pnum = 2; _pltype = 6|1; _plwidth = 0|0;
    _pnumht = 0.25; _ptitlht = 0.25; _paxht = 0.25;
    fonts("simplex simgrma complex");
    _pline = 1~5~CouponZero~0~CouponZero~100~1~15~0;
    ytics(0,6,1,0);
    xtics(0.7,1.2,0.1,0);

  setwind(2);
 
    title("k = 0.04");
    _plegstr = "\203Trapeze\000\203Simpson";
    _plegctl = {2 8 7 1};
    xy(x1a~x1b,d1a~d1b);

  setwind(3);

    title("k = 0.02");
    _plegctl = {2 8 -2 1};
    xy(x2a~x2b,d2a~d2b);

  setwind(4);

    title("k = 0.01");
    _plegctl = 0;
    xy(x3a~x3b,d3a~d3b);

  setwind(5);

    title("k = 0.004");
    xy(x4a~x4b,d4a~d4b);

    graphprt("-c=1 -cf=mc1.eps");
  endwind;
