new;
library spt,pgraph;
SPTset;

cls;

rndseed 123;

a = 0.95; b = 0.10; sigma = 0.2; lambda = 0.05;
theta = 0.5;

bprime = b - lambda*sigma/a;


proc (2) = Theta_Euler(r0,t0,T,N,M,theta);
  local k,tau,u,S,Si,w1,w2,w3,i;

  k = (T - t0)/(N-1);
  tau = seqa(0,k,N);
  u = rndn(N,M)*sqrt(k);
  S = zeros(N,M);

  Si = r0.*ones(1,M);
  S[1,.] = Si;

  w1 = 1 + theta*a*k;
  w2 = 1 - (1-theta)*a*k;
  w3 = a*k*bprime;

  i = 2;
  do until i > N;
    Si = ( w2*Si + w3 + sigma*u[i,.] )/w1;
    S[i,.] = Si;
    i = i + 1;
  endo;

  retp(tau,S);
endp;


proc intTrapeze2(S,k);
  local N,M,I,j;

  N = rows(S); M = cols(S);
  I = zeros(N,M);

  j = 2;
  do until j > N;
    I[j,.] = I[j-1,.] + 0.5*(S[j-1,.] + S[j,.])*k;
    j = j + 1;
  endo;

  retp(I);
endp;


t0 = 0;
T = 1;
r0 = 0.10;

N = 101;
k = (T-t0)/(N-1);


/* 100 simulations */

M1 = 100; 
M2 = 1;

j = 1;
do until j > M2;

  {tau,S} = Theta_Euler(r0,t0,T,N,M1,theta);
  P1 = meanc(exp(-intTrapeze2(S,k)'));

  j = j + 1;
endo;

/* 1000 simulations */

M1 = 1000; 
M2 = 1;

j = 1;
do until j > M2;

  {tau,S} = Theta_Euler(r0,t0,T,N,M1,theta);
  P2 = meanc(exp(-intTrapeze2(S,k)'));

  j = j + 1;
endo;

/* 10000 simulations */

M1 = 1000; 
M2 = 10;
P3 = 0;

j = 1;
do until j > M2;

  {tau,S} = Theta_Euler(r0,t0,T,N,M1,theta);
  P3 = P3 + meanc(exp(-intTrapeze2(S,k)'));

  j = j + 1;
endo;

P3 = P3/M2;


/* 500 000 simulations */

M1 = 1000; 
M2 = 500;
M = M1 + M2;
P4 = 0;

j = 1;
do until j > M2;

  {tau,S} = Theta_Euler(r0,t0,T,N,M1,theta);
  P4 = P4 + meanc(exp(-intTrapeze2(S,k)'));

  locate 5,5;
  print /flush ftos(j,"Iteration no. %lf",2,0);

  j = j + 1;
endo;

P4 = P4/M2;


{CouponZero,TauxZero} = Vasicek(r0,a,b,sigma,lambda,tau);

graphset;
  begwind;
  window(2,2,0);
    _pdate = ""; _pnum = 2; _pnumht = 0.25; _paxht = 0.3; _ptitlht = 0.25;
    fonts("simplex simgrma");
    xlabel("\202t\201");
    xtics(0,1,0.2,2);
    ylabel("erreur");
    _pysci = 0;

  setwind(1);
    title("M = 100");
    ytics(0,0.006,0.001,0);
    xy(tau,CouponZero-P1);
  setwind(2);
    title("M = 1 000");
    ytics(-0.0003,0.0006,0.0002,0);
    xy(tau,CouponZero-P2);
  setwind(3);  
    title("M = 10 000");
    ytics(-0.0009,0,0.0002,0);
    xy(tau,CouponZero-P3);
  setwind(4);
    title("M = 500 000");
    ytics(-0.00009,-0.00001,0.00002,0);
    xy(tau,CouponZero-P4);

    graphprt("-c=1 -cf=mc3.eps");
  endwind;