new;
library tsm,optmum,spt,pgraph;
TSMset;

cls;

rndseed 123456;

proc Solution(alpha,X0,MU,SIGMA,Rho,Lambda,tau);
  local r0,w1,w2,CouponZero;

  r0 = alpha'X0;
  w1 = alpha'(MU-SIGMA*LAMBDA);
  w2 = alpha'*SIGMA*Rho*SIGMA'alpha;
  
  CouponZero = exp( -R0.*tau - 0.5*w1.*(tau^2) + w2.*(tau^3)/6 );

  retp(CouponZero);
endp;


Proc (4) = SimulationTaylor(X0,Muproc,SIGMAproc,Rho,t0,T,NBpoints);
  local Muproc:proc,SIGMAproc:proc;
  local M,N,X1,X2,k,tt,Pchol,X1i,X2i,ti,i,e;

  M = rows(X0);  /* Nombre de variables d'etat */
  N = rows(Rho); /* Nombre de risques          */

  X1 = zeros(M,NBpoints);
  X1[.,1] = X0;
  X2 = zeros(M,NBpoints);
  X2[.,1] = X0;
  
  k = (T-t0)/(NBpoints-1);
  tt = seqa(t0,k,NBpoints);

  Pchol = chol(Rho)';
  Pchol = sqrt(k)*Pchol;
  
  X1i = X0;
  X2i = X0;
  ti = t0;

  i = 2;
  do until i > NBpoints;
    e = rndn(N,1);
    X1i = X1i + MUproc(ti,X1i)*k + SIGMAproc(ti,X1i)*Pchol*e;
    X2i = X2i + MUproc(ti,X2i)*k - SIGMAproc(ti,X2i)*Pchol*e;
    X1[.,i] = X1i;
    X2[.,i] = X2i;
    ti = tt[i];
    i = i + 1;
  endo;

  retp(k,tt,X1',X2');
endp;

proc intTrapeze(S,k);
  local N,I;

  N = rows(S);
  I = k*(sumc(S[2:N-1,.]) + 0.5*(S[1,.]' + S[N,.]') );

  retp(I);
endp;

let X0 = 0.10 0.15;
let MU = 0.3 0.5;
let SIGMA[2,2] = 0.25 0 0 0.15;
let Rho[2,2] = 1 0.5 0.5 1;
let Lambda = 0.1 0.2;
let alpha = 0.3 -0.1;

proc pMU(t,X);
  retp(MU-SIGMA*Lambda);
endp;

proc pSIGMA(t,X);
  retp(SIGMA);
endp;

proc pTaux(t,X);
  retp(alpha*X);
endp;


Nr = 250;

Ns = 10;

CZ1 = zeros(Ns*Nr,1);
CZ2 = zeros(Ns*Nr,1);

s = 1;
do until s > Ns*Nr;
  {k,ts,X1s,X2s} = SimulationTaylor(X0,&pMu,&pSIGMA,Rho,0,5,100);
  R1s = X1s*alpha;
  R2s = X2s*alpha;
  CZ1[s] = exp(-intTrapeze(R1s,k));
  CZ2[s] = 0.5*(exp(-intTrapeze(R1s,k))+exp(-intTrapeze(R2s,k)));
  locate 5,5; print ftos(s,"1. Iteration No. %lf",2,0);
  s = s + 1;
endo;
  
CZ1 = meanc(reshape(CZ1,Ns,Nr));
CZ2 = meanc(reshape(CZ2,Ns,Nr));

Ns = 50;

CZ3 = zeros(Ns*Nr,1);
CZ4 = zeros(Ns*Nr,1);

s = 1;
do until s > Ns*Nr;
  {k,ts,X1s,X2s} = SimulationTaylor(X0,&pMu,&pSIGMA,Rho,0,5,100);
  R1s = X1s*alpha;
  R2s = X2s*alpha;
  CZ3[s] = exp(-intTrapeze(R1s,k));
  CZ4[s] = 0.5*(exp(-intTrapeze(R1s,k))+exp(-intTrapeze(R2s,k)));
  locate 5,5; print ftos(s,"2. Iteration No. %lf",2,0);
  s = s + 1;
endo;
  
CZ3 = meanc(reshape(CZ3,Ns,Nr));
CZ4 = meanc(reshape(CZ4,Ns,Nr));

CZ = Solution(alpha,X0,MU,SIGMA,Rho,Lambda,5);



_Kernel[1] = 0.45;
_Kernel[2] = 0.80;

{x1,d1,F1,retcode} = Kernel(CZ1);
{x2,d2,F2,retcode} = Kernel(CZ2);
{x3,d3,F3,retcode} = Kernel(CZ3);
{x4,d4,F4,retcode} = Kernel(CZ4);


graphset;
  _pdate = ""; _pnum = 2;
  title("Acceleration de Monte Carlo");
  _plegstr = "S = 10\000S = 10 + acc.\000S = 50\000S = 50 + acc.";
  _plegctl = {2 5 1 3};
  _pltype = 6|1|6|1;
  _pstype = 8|12|8|12;
  _plctrl = 10|5|10|2;
  _pline = 1~5~CZ~0~CZ~5000~1~15~0;
  xtics(0.5,0.8,0.1,10);
  graphprt("-c=1 -cf=mc6.eps");
  xy(x1~x2~x3~x4,d1~d2~d3~d4);


