new;
library tsm,optmum,spt,pgraph;
TSMset;

cls;

rndseed 456;

/*
** Schema de simulation de Taylor a l'ordre 0.5
*/

proc (2) = simulation1(x0,a,b,sigma,t0,TT,N,Ns);
  local k,t,x,u,k1,i,xi;

  k = (TT-t0)/(N-1);
  t = seqa(t0,k,N);
  x = zeros(N,Ns);

  x[1,.] = x0.*ones(1,Ns);

  u = rndn(N,Ns);
  k1 = sqrt(k);

  i = 1;
  do until i > N-1;
    xi = x[i,.];
    x[i+1,.] = xi + a*(b-xi)*k + sigma*k1*u[i+1,.];
    i = i + 1;
  endo;

  retp(t,x);
endp;


/*
** Schema de simulation de Taylor a l'ordre 1.5
*/

proc (2) = simulation2(x0,a,b,sigma,t0,TT,N,Ns);
  local k,t,x,e1,e2,u1,u2,i,xi;
  local alpha,beta,Dalpha,u1i,u2i,k1;

  k = (TT-t0)/(N-1);
  t = seqa(t0,k,N);
  x = zeros(N,Ns);

  x[1,.] = x0.*ones(1,Ns);

  e1 = rndn(N,Ns);
  e2 = rndn(N,Ns);
  u1 = e1*sqrt(k);
  u2 = 0.5*(k^(3/2))*(e1+e2/sqrt(3));
  k1 = k^2;

  i = 1;
  do until i > N-1;
    u1i = u1[i,.]; 
    u2i = u2[i,.]; 
    xi = x[i,.];
    alpha = a*(b-xi); 
    beta = sigma; 
    Dalpha = -a;
    x[i+1,.] = xi + alpha*k + beta*u1i + Dalpha*beta*u2i
                  + 0.5*(Dalpha*alpha)*k1;
    i = i + 1;
  endo;

  retp(t,x);
endp;


x0 = 0;
a = 0.8;
b = 0.1;
sigma = 0.06;
k = 1|0.5|0.1;
sv = a|b|sigma;

Nr = 250;  /* Nombre de replications */

__output = 0;
_print = 0;
_tsm_Mcov = 0;

Nobs = 1000;
Est1000_a = zeros(Nr,3);
Est1000_b = zeros(Nr,3);
Est1000_sigma = zeros(Nr,3);


j = 1;
do until j > 3;

  i = 1;
  do until i > Nr;
    {t,x} = simulation2(x0,a,b,sigma,0,Nobs*k[j],Nobs,1);
    {theta,stderr,Mcov,Logl} = ml_OU(x,k[j],sv,0,0);
    str = "1000 observations --- " $+ ftos(i,"Replication %lf",2,0) $+
                          ftos(j," --- j = %lf",3,2);
    locate 5,5; print str;
    Est1000_a[i,j] = theta[1];
    Est1000_b[i,j] = theta[2];
    Est1000_sigma[i,j] = theta[3];   
    i = i + 1;
  endo;

  j = j + 1;
endo;


save simul11 = Est1000_a;
save simul12 = Est1000_b;
save simul13 = Est1000_sigma;


load Est1000_a = simul11;
load Est1000_b = simul12;
load Est1000_sigma = simul13;


X1 = {};
D1 = {};

_kernel[1:2] = 0.50|1.50;

i = 1;
do until i > 3;

  {xi,di,Fi,retcode} = Kernel(Est1000_a[.,i]);
  X1 = X1~xi;
  D1 = D1~di;

  i = i + 1;
endo;


X2 = {};
D2 = {};

_kernel[1:2] = 0.08|0.12;

i = 1;
do until i > 3;

  {xi,di,Fi,retcode} = Kernel(Est1000_b[.,i]);
  X2 = X2~xi;
  D2 = D2~di;

  i = i + 1;
endo;


X3 = {};
D3 = {};

_Kernel[1:2] = 0.04|0.07;

i = 1;
do until i > 3;

  {xi,di,Fi,retcode} = Kernel(abs(Est1000_sigma[.,i]));
  X3 = X3~xi;
  D3 = D3~di;

  i = i + 1;
endo;


graphset;
  begwind;
  window(2,2,0);

  setwind(1);

    _pdate = ""; _pnum = 0; _paxes = 0; _ptitlht = 0.30;
    fonts("simplex simgrma complex microb");
    _pmsgstr = "\203Densite empirique des estimateurs de\000"\
                   "la methode du maximum de vraisemblance\000"\
                   "pour le processus d'Ornstein-Uhlenbeck\000"\
                   "schema de Taylor a l'ordre 1.5";
           
    _pmsgctl = 
               0.05~0.75~0.23~0~1~7~0 |
               0.05~0.6~0.23~0~1~7~0 |
               0.05~0.45~0.23~0~1~7~0 |
               0.05~0.2~0.23~0~1~7~0 ;            
    draw;

  graphset;
    _pnum = 2; _pdate = ""; _pnumht = 0.25; _ptitlht = 0.25; 
    _pltype = 6|1|3|4;
    fonts("simplex simgrma");
  
  setwind(2);

    xtics(0.4,1.4,0.2,0);
    ytics(0,8,1,0);
    title("\201parametre a");
    _pline = 1~6~0.8~0~0.8~1000~1~7~0;
    xy(X1,D1);

  setwind(3);
    xtics(0.08,0.12,0.01,0);
    ytics(0,150,50,0);
    title("\201parametre b");
    _pline = 1~6~0.10~0~0.10~1000~1~7~0;
    _plegstr = "k = 1\0k = 0.5\0k = 0.1";
    _plegctl = {2 7 7.5 2};
    xy(X2,D2);

  setwind(4);

    xtics(0.04,0.07,0.01,0);
    ytics(0,300,50,0);
    title("\201parametre \202s\201");
    _pline = 1~6~0.06~0~0.06~1000~1~7~0;
    _plegctl = {2 7 -1.5 2};
    xy(X3,D3);

    graphprt("-c=1 -cf=ou4.eps");
  endwind;


