new;
library spt,pgraph,tsm,optmum;
TSMset;

cls;

sigma1 = 0.02;
sigma2 = 0.05;
c = 0.125;

proc procSigma(theta,T);
  local Nobs;
  Nobs = maxc(rows(theta)|rows(T));
  retp( sigma1*ones(Nobs,1) ~ sigma2.*exp(-c/2.*(T-theta)) );
endp;

proc Solution(t0,t);
  local dt,sol1,sol2;
  dt = t - t0;
  sol1 = 0.5*(sigma1*dt)^2;
  sol2 = -2*(sigma2/c)^2*( (1-exp(-c*dt)) - 2*(1-exp(-c/2*dt)) );
  retp(sol1 ~ sol2);
endp;

N = 5|10|25|50;
ch = "sol";

j = 1;
do until j > 4;

  t = seqa(1,0.5,140);
  res = zeros(140,4);

  i = 1;
  do until i > 140;
    res[i,.] = intsimpHJM(&procSigma,1,t[i],N[j]) ~ Solution(1,t[i]);
    i = i + 1;
    locate 5,5; 
    print /flush ftos(j,"j = %lf",2,0)$+ftos(i," --- i = %lf",5,0);
  endo;

  call varput(res,ch$+ftos(j,"%lf",1,0));

  j = j + 1;
endo;



graphset;
  begwind;
  window2(2,2,0,2);

  setwind(1);

    _pdate = ""; _pnum = 0; _paxes = 0;
    fonts("simplex simgrma");

    _pmsgstr = "Etude de l'algorithme numerique de Simpson pour le calcul de"\
               "\000\202\124\201[t]]t]0[[\202s\34\201u,t\202\118"\
               " @[\124\201[t]]u[\202s\34\201u,v\202\118 \201dv\202@]\201 du"\
               "\000application a la fonction"\
               "\000\202s\34Q\201,T\202\118\201 = 0.05\202*"\
               "\201exp\202\34\201-0.125/2\202*\34\201T-\202Q\118\118\201";
           
    _pmsgctl = 
               0.05~0.8~0.5~0~1~7~0 |
               0.5~0.6~0.6~0~1~7~0 |              
               0.05~0.2~0.5~0~1~7~0 |            
               0.4~0.2~0.6~0~1~7~0 ;            
    draw;

    graphset;
    _paxht = 0.30; _pnum = 2; _pnumht = 0.35; _ptitlht = 0.30;
    _pysci = 0; 
    xtics(0,60,20,0);
    xlabel("t");
  
  setwind(2);

    title("5 points de discretisation");

    _plegstr = "Solution numerique\0Solution symbolique";
    _plegctl = { 2 7 6 2};
    ytics(0,0.32,0.08,0); 
    r = varget("Sol1");
    xy(t,r[.,2 4]);

  nextwind;
  
    title("10 points de discretisation");  
    _plegctl = { 2 7 -3 2};
    r = varget("Sol2");
    xy(t,r[.,2 4]);

  nextwind;

    title("25 points de discretisation");  
    ylabel("erreur");
    _plegctl = 0;
    _pyscale = 0;
    ytics(0,0.004,0.001,0);
    r = varget("Sol3");
    xy(t,r[.,4]-r[.,2]);

  nextwind;

    title("50 points de discretisation");    
    ytics(0,0.0000008,0.0000002,0);
    r = varget("Sol4");
    xy(t,r[.,4]-r[.,2]);

  graphprt("-c=1 -cf=hjm1.eps");

  endwind;

